from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from django.db import connection
from datetime import date
from django.utils.safestring import mark_safe


@login_required
def overview(request):
    """
    View for the overview of the target-system
    :param request:
    :return: redered page
    """
    today = date.today()
    res = _get_data_dict(today.month, today.year)
    if today.month == 1:
        year_before = today.year -1
        month_before = 12
    else:
        year_before = today.year
        month_before = today.month - 1
    res_before = _get_data_dict(month_before, year_before)
    rvalues = {"rows": res, "month": today.month, "year": today.year}
    rvalues["month_before"] = month_before
    rvalues["year_before"] = year_before
    rvalues["rows_before"] = res_before
    return render(request, "target/main.html", rvalues)


@login_required
def detailed(request, organisation_id, year):
    res = [_get_all_data_dict(k, year, organisation_id) for k in range(1,13)]
    # for display grey font color in future.
    for d in res:
        if not d.get("color"):
            d["futurestyle"] = mark_safe('style="color: darkgrey;"')
        else:
            d["futurestyle"]  = ""
    max_year = min(date.today().year, 2020)
    return render(request, "target/detail.html",
            {"id": organisation_id, "year": year, "data": res,
             "years": range(2015, max_year + 1)})


def _get_data_dict(month, year):
    cursor = connection.cursor()
    cursor.execute("SELECT organisation_id, quota, color, total_quota, "
                   "total_color FROM eval_targets WHERE year = %s AND "
                   "month = %s;", (str(year), str(month)))
    columns = [col[0] for col in cursor.description]
    return [dict(zip(columns, row)) for row in cursor.fetchall()]


def _get_all_data_dict(month, year, organisation_id):
    cursor = connection.cursor()
    cursor.execute("SELECT month, organisation_id,"
                   "target, entries, quota, color,"
                   "cum_target, cum_entries, cum_quota, cum_color,"
                   "total_target, total_entries, total_quota, total_color"
                   " FROM eval_targets WHERE year = %s AND "
                   "month = %s AND organisation_id = %s;",
                   (str(year), str(month), str(organisation_id)))
    columns = [col[0] for col in cursor.description]
    line = cursor.fetchone()
    if line:
        return dict(zip(columns, line))
    else:
        return dict()
