import datetime
from collections import namedtuple

import re
import reportserver.lib.validators as validators

Filteroptions = namedtuple("Filteroptions", "start end nuts1 regioncategory beneficiary gender filter_type foerderphase")


def check_date(date):
    """
    Savely retrieve Date gracefully or fall back to default
    :param request: request
    :param name: name of request parameter
    :param default: default date
    :return:
    """
    try:
        datetime.datetime.strptime(date, '%Y-%m-%d')
    except:
        raise ValueError("Wrong dateformat")
    return date


def convert_german_date_to_iso(datestring):
    """
    changes german date to iso format
    if string is not german format, it is returned
    :param datestring:
    :return:
    """
    p = re.compile(r'^\d{1,2}\.\d{1,2}\.\d{1,4}$')
    if p.match(datestring):
        day, month, year = datestring.split(".")
        datestring = "%s-%s-%s" % (year, month, day)
    return datestring


def allowed(request, name):
    """
    get values known to DB
    :param request:
    :param name:
    :return:
    """
    return (entry[0] for entry in request.session.get(name))


def filter_empty(elements):
    return [e for e in elements if e]


def get_selected_options_from(request):
    """
    Gets the filter options from request
    independently whether it is a GET or POST
    :param request:
    :return:
    """
    nuts1 = filter_empty(request.POST.getlist("nuts1"))
    beneficiary = filter_empty(request.POST.getlist("beneficiary"))
    regioncategory = filter_empty(request.POST.getlist("regioncategory"))
    start_date = check_date(convert_german_date_to_iso(request.POST.get('startdate')))
    end_date = check_date(convert_german_date_to_iso(request.POST.get('enddate')))
    gender = filter_empty(request.POST.getlist("gender"))
    filter_type = request.POST.get('date_filter_type')
    foerderphase = filter_empty(request.POST.getlist("foerderphase"))
    validators.check_nuts1(nuts1, allowed(request, "nuts1"))
    validators.check_beneficiary(beneficiary, allowed(request, "beneficiary"))
    validators.check_regioncategory(regioncategory, allowed(request, "regioncategory"))
    validators.check_gender(gender)

    validators.check_filter_type(filter_type)
    validators.check_foerderphase(foerderphase)

    return Filteroptions(start_date, end_date, nuts1, regioncategory,
                         beneficiary, gender, filter_type, foerderphase)
